# generated by datamodel-codegen:
#   filename:  openapi.yaml

from __future__ import annotations

from datetime import date
from enum import Enum
from typing import Any, List, Optional

from pydantic import Field

from cognite.well_model.base_model import WellsBaseModel


class SummaryCount(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    property: str
    count: Optional[int] = None


class CursorAndLimit(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    limit: Optional[int] = Field(
        None,
        description="Maximum number of rows returned in one request. If there is more data beyond this limit, a cursor will be returned to enable further fetching of data.",
        ge=1.0,
        le=1000.0,
    )
    cursor: Optional[str] = Field(
        None,
        description="Cursor for pagination returned from a previous request. Apart from cursor and limit, the rest of the request object must be the same as for the original request.",
    )


class StringItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[str]


class DateRange(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    min: Optional[date] = None
    max: Optional[date] = None


class DistanceUnitEnum(str, Enum):
    meter = "meter"
    foot = "foot"
    inch = "inch"
    yard = "yard"


class AngleUnitEnum(str, Enum):
    degree = "degree"
    radian = "radian"


class WeightUnitEnum(str, Enum):
    kilogram = "kilogram"
    pound = "pound"


class DurationUnitEnum(str, Enum):
    millisecond = "millisecond"
    second = "second"
    minute = "minute"
    hour = "hour"
    day = "day"


class DepthIndexTypeEnum(str, Enum):
    measured_depth = "measured depth"
    true_vertical_depth = "true vertical depth"


class DurationRange(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    unit: DurationUnitEnum
    min: Optional[float] = None
    max: Optional[float] = None


class Identifier(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    matching_id: Optional[str] = Field(None, alias="matchingId", description="Matching id of a well or a wellbore.")
    asset_external_id: Optional[str] = Field(
        None, alias="assetExternalId", description="Asset id of a well or a wellbore"
    )


class IdentifierItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Identifier] = Field(..., max_items=1000, min_items=1)
    ignore_unknown_ids: Optional[bool] = Field(None, alias="ignoreUnknownIds")


class SequenceExternalId(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    sequence_external_id: str = Field(
        ..., alias="sequenceExternalId", description="An external id for a sequence in CDF."
    )


class EventExternalId(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    event_external_id: str = Field(..., alias="eventExternalId")


class ExternalIdTypeEnum(str, Enum):
    asset = "asset"
    event = "event"
    file = "file"
    sequence = "sequence"
    timeseries = "timeseries"


class SummaryItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[SummaryCount]


class WellPropertiesEnum(str, Enum):
    region = "region"
    country = "country"
    block = "block"
    field = "field"
    quadrant = "quadrant"
    operator = "operator"


class WellPropertiesSummaryRequest(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellPropertiesEnum]


class WellPropertiesSummaryRow(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    region: Optional[str] = None
    country: Optional[str] = None
    block: Optional[str] = None
    field: Optional[str] = None
    quadrant: Optional[str] = None
    operator: Optional[str] = None
    wells_count: int = Field(..., alias="wellsCount")


class PropertyFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    is_set: Optional[bool] = Field(None, alias="isSet")
    one_of: Optional[List[str]] = Field(None, alias="oneOf")


class WellMergeRules(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: List[str]
    description: List[str]
    country: List[str]
    quadrant: List[str]
    region: List[str]
    block: List[str]
    field: List[str]
    operator: List[str]
    spud_date: List[str] = Field(..., alias="spudDate")
    license: List[str]
    well_type: List[str] = Field(..., alias="wellType")
    water_depth: List[str] = Field(..., alias="waterDepth")
    wellhead: List[str]


class Wellhead(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    x: float = Field(..., example=1)
    y: float = Field(..., example=1)
    crs: str = Field(..., example="Example")


class WellboreMergeRules(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: List[str]
    description: List[str]
    datum: List[str]
    parents: List[str]
    well_tops: List[str] = Field(..., alias="wellTops")


class SpudDateLimits(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    min: Optional[date] = None
    max: Optional[date] = None


class DeleteEventSources(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[EventExternalId] = Field(..., max_items=1000, min_items=1)


class NptAggregateEnum(str, Enum):
    npt_code = "npt code"
    npt_code_detail = "npt code detail"


class NptAggregateRequestFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_ids: List[Identifier] = Field(..., alias="wellboreIds", max_items=1000, min_items=1)


class ContainsAllOrAny(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    contains_all: Optional[List[str]] = Field(None, alias="containsAll", description="match on all of the strings")
    contains_any: Optional[List[str]] = Field(None, alias="containsAny", description="match on any of the strings")


class ContainsAllOrAnyInt(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    contains_all: Optional[List[int]] = Field(None, alias="containsAll", description="match on all of the strings")
    contains_any: Optional[List[int]] = Field(None, alias="containsAny", description="match on any of the strings")


class NdsAggregateEnum(str, Enum):
    severity = "severity"
    probability = "probability"
    risk_type = "risk type"
    subtype = "subtype"


class NdsAggregateRequestFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_ids: List[Identifier] = Field(..., alias="wellboreIds", max_items=1000, min_items=1)


class NdsAggregateRow(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    count: int = Field(..., description="Number of NPT events that matches this group")
    severity: Optional[int] = None
    probability: Optional[int] = None
    risk_type: Optional[str] = Field(None, alias="riskType")
    subtype: Optional[str] = None


class DepthMeasurementColumn(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measurement_type: str = Field(..., alias="measurementType")
    column_external_id: str = Field(
        ..., alias="columnExternalId", description="External id of the sequence column", example="GAMMA_RAY"
    )
    unit: str = Field(..., description="Unit for the measurement type", example="gApi")
    description: Optional[str] = None


class MeasurementType(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    type: Optional[str] = None


class TimeMeasurementColumn(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    timeseries_external_id: str = Field(
        ...,
        alias="timeseriesExternalId",
        description="The CDF external id of the timeseries containing the measurement data",
    )
    unit: str
    measurement_type: str = Field(..., alias="measurementType")


class TimeMeasurementFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measurement_types: Optional[List[str]] = Field(None, alias="measurementTypes")
    wellbore_ids: Optional[List[Identifier]] = Field(None, alias="wellboreIds", max_items=1000, min_items=1)


class TrajectoryIngestionRow(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measured_depth: float = Field(..., alias="measuredDepth")
    inclination: float
    azimuth: float
    dogleg_severity: Optional[float] = Field(None, alias="doglegSeverity")


class TrajectoryFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_ids: Optional[List[Identifier]] = Field(None, alias="wellboreIds", max_items=1000, min_items=1)
    is_definitive: Optional[bool] = Field(
        None, alias="isDefinitive", description="If set to true, you will only retrieve definitive trajectories."
    )


class TrajectoryFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[TrajectoryFilter] = None


class DeleteSequenceSources(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[SequenceExternalId] = Field(..., max_items=1000, min_items=1)


class Chronostratigraphic(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    age: Optional[str] = Field(
        None, description="Geochronologic age/stage. Examples: maastrichtian, danian, lutetian.", example="kasimovian"
    )
    sub_epoch: Optional[str] = Field(
        None,
        alias="subEpoch",
        description="Geochronologic sub epoch used in parts of north america. Example: upper pennsylvanian.",
        example="upper pennsylvanian",
    )
    epoch: Optional[str] = Field(
        None, description="Geochronologic epoch. Examples: miocene, eocene, upper jurassic.", example="pennsylvanian"
    )
    period: Optional[str] = Field(
        None, description="Geochronologic period. Examples. neogene, cretaceous, jurassic.", example="carboniferous"
    )
    era: Optional[str] = Field(
        None, description="Geochronologic era: Examples: Cenozoic, Mesozoic.", example="paleozoic"
    )


class LithostratigraphicLevelEnum(str, Enum):
    bed = "bed"
    member = "member"
    formation = "formation"
    group = "group"


class Lithostratigraphic(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    level: LithostratigraphicLevelEnum


class WellTopSurfaceDepth(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measured_depth: float = Field(..., alias="measuredDepth")
    true_vertical_depth: Optional[float] = Field(None, alias="trueVerticalDepth")


class WellTopsFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_ids: List[Identifier] = Field(
        ..., alias="wellboreIds", description="Limit results to following wellbores", max_items=1000, min_items=1
    )


class DistanceRange(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    unit: DistanceUnitEnum
    min: Optional[float] = Field(None, description="Lower bound of the range.")
    max: Optional[float] = Field(None, description="Upper bound of the range.")


class AngleRange(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    unit: AngleUnitEnum
    min: Optional[float] = None
    max: Optional[float] = None


class TimeRange(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    min: Optional[int] = Field(None, description="epoch time stamp in milliseconds when time range started.", example=1)
    max: Optional[int] = Field(None, description="epoch time stamp in milliseconds when time range ended.", example=1)


class TrajectoryDataRequest(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    sequence_external_id: str = Field(..., alias="sequenceExternalId")
    measured_depth: Optional[DistanceRange] = Field(None, alias="measuredDepth")
    true_vertical_depth: Optional[DistanceRange] = Field(None, alias="trueVerticalDepth")


class TrajectoryDataRow(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    true_vertical_depth: float = Field(..., alias="trueVerticalDepth")
    measured_depth: float = Field(..., alias="measuredDepth")
    northing: Optional[float] = None
    easting: Optional[float] = None
    north_offset: float = Field(..., alias="northOffset")
    east_offset: float = Field(..., alias="eastOffset")
    azimuth: float
    inclination: float
    dogleg_severity: float = Field(..., alias="doglegSeverity")
    equivalent_departure: float = Field(..., alias="equivalentDeparture")


class TrajectoryDataRequestItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[TrajectoryDataRequest]


class CasingFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_ids: Optional[List[Identifier]] = Field(None, alias="wellboreIds", max_items=1000, min_items=1)


class CasingFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[CasingFilter] = None


class WellSearch(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    query: str = Field(..., description="Fuzzy search on well name")


class WellFilterAggregatesEnum(str, Enum):
    count = "count"


class GeometryTypeEnum(str, Enum):
    wkt = "WKT"
    geo_json = "GeoJSON"


class WellTopFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    exists: Optional[bool] = Field(
        True,
        description="Set to false to filter on wellbores that do not pass through the specified surfaces. If no surfaces are specified then only wellbores that are not associated with any surface are returned.\n",
    )
    surface_names: Optional[ContainsAllOrAny] = Field(None, alias="surfaceNames")


class ContainsAllOrAnyMeasurementType(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    contains_any: List[str] = Field(..., alias="containsAny")


class WellCasingFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    exists: Optional[bool] = Field(
        True, description="Set to false to filter on wellbores that do not have any casings."
    )


class WellDataAvailabilityFilterEnum(str, Enum):
    npt = "npt"
    nds = "nds"
    trajectories = "trajectories"
    depth_measurements = "depthMeasurements"
    casings = "casings"
    hole_sections = "holeSections"
    time_measurements = "timeMeasurements"
    well_tops = "wellTops"


class WellDataAvailabilityFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    __root__: List[WellDataAvailabilityFilterEnum] = Field(
        ...,
        description="This filter will only return wells that have at least one element of any type from the list provided",
        example=["npt", "nds", "casings", "trajectories"],
        min_items=1,
    )


class DistanceUnit(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    unit: DistanceUnitEnum
    factor: Optional[float] = Field(1.0, example=1.0)


class DepthMeasurementRow(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    row_number: int = Field(..., alias="rowNumber", description="The row number for this row", example=1, ge=0.0)
    depth: float
    values: List[Any] = Field(
        ...,
        description="List of values in order defined in the columns field (Number of items must match. Null is accepted for missing values. String values must be no longer than 256 characters)",
        max_items=200,
        min_items=1,
    )


class DepthMeasurementValueTypeEnum(str, Enum):
    string = "string"
    double = "double"
    long = "long"


class AssetSource(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    asset_external_id: str = Field(..., alias="assetExternalId")
    source_name: str = Field(..., alias="sourceName", example="EDM")


class EventSource(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    event_external_id: str = Field(..., alias="eventExternalId")
    source_name: str = Field(..., alias="sourceName")


class SequenceSource(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    sequence_external_id: str = Field(..., alias="sequenceExternalId", description="An external id of a CDF sequence.")
    source_name: str = Field(..., alias="sourceName", description="Name of the source.", example="EDM")


class ExternalIdSource(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    external_id: str = Field(..., alias="externalId", description="An external id of a resource.")


class Error(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    code: int
    message: str


class ErrorResponse(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    error: Error


class Source(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: str = Field(..., example="EDM")
    description: Optional[str] = None


class MnemonicSearch(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    mnemonics: List[str]


class MnemonicMatchTool(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    code: str
    description: Optional[str] = None


class MeasurementTypeFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measurement_types: List[str] = Field(..., alias="measurementTypes", max_items=1000, min_items=1)


class MeasurementTypeDetails(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    property: str
    path: str
    description: str
    primary_quantity_class: str = Field(..., alias="primaryQuantityClass")


class Distance(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    value: float = Field(..., description="Amount of a given unit.")
    unit: DistanceUnitEnum


class Duration(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    value: float
    unit: DurationUnitEnum


class Angle(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    value: float
    unit: AngleUnitEnum


class DoglegSeverityUnit(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    angle_unit: AngleUnitEnum = Field(..., alias="angleUnit")
    distance_unit: DistanceUnitEnum = Field(..., alias="distanceUnit")
    distance_interval: Optional[float] = Field(30, alias="distanceInterval")


class SequenceExternalIdItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[SequenceExternalId] = Field(..., max_items=1000, min_items=1)
    ignore_unknown_ids: Optional[bool] = Field(False, alias="ignoreUnknownIds")


class GeneralExternalId(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    external_id: str = Field(..., alias="externalId", description="An identifier for a CDF resource.")
    type: ExternalIdTypeEnum


class WellPropertiesSummaryItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellPropertiesSummaryRow]


class WellIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    matching_id: Optional[str] = Field(None, alias="matchingId")
    name: str
    description: Optional[str] = Field(
        None, description="additional description for the data", example="extracted from EDM"
    )
    country: Optional[str] = Field(None, example="Norway")
    quadrant: Optional[str] = Field(None, example="25")
    region: Optional[str] = None
    spud_date: Optional[date] = Field(None, alias="spudDate")
    block: Optional[str] = Field(None, example="25/5")
    field: Optional[str] = Field(None, example="Example")
    operator: Optional[str] = Field(None, example="Op1")
    well_type: Optional[str] = Field(None, alias="wellType", description='For example "exploration" or "development"')
    license: Optional[str] = Field(None, description="Well license")
    water_depth: Optional[Distance] = Field(None, alias="waterDepth")
    wellhead: Optional[Wellhead] = None
    source: AssetSource


class DeleteWells(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[AssetSource] = Field(..., max_items=1000, min_items=1)
    recursive: Optional[bool] = Field(
        False,
        description="If set to true, it will delete associated wellbores, measurements, NPT/NDS etc.\nIf set to false, it will fail if one of the wells have associated wellbores.",
    )


class WellWellheadView(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    well_matching_id: str = Field(..., alias="wellMatchingId")
    wellhead: Wellhead


class FieldSources(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: Optional[AssetSource] = None
    description: Optional[AssetSource] = None
    country: Optional[AssetSource] = None
    quadrant: Optional[AssetSource] = None
    spud_date: Optional[AssetSource] = Field(None, alias="spudDate")
    block: Optional[AssetSource] = None
    field: Optional[AssetSource] = None
    operator: Optional[AssetSource] = None
    well_type: Optional[AssetSource] = Field(None, alias="wellType")
    license: Optional[AssetSource] = None
    water_depth: Optional[AssetSource] = Field(None, alias="waterDepth")
    wellhead: Optional[AssetSource] = None


class IncompleteWell(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    matching_id: str = Field(..., alias="matchingId")
    sources: List[AssetSource]
    missing: List[str]


class IncompleteWellItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[IncompleteWell]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class FieldSources1(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: Optional[AssetSource] = None
    description: Optional[AssetSource] = None
    datum: Optional[AssetSource] = None


class IncompleteWellbore(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    matching_id: str = Field(..., alias="matchingId")
    sources: List[AssetSource]
    missing: List[str]


class IncompleteWellboreItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[IncompleteWellbore]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class WaterDepthLimits(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    min: Optional[Distance] = None
    max: Optional[Distance] = None


class Npt(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ..., alias="wellboreAssetExternalId", description="The CDF asset external id to the connected wellbore."
    )
    wellbore_matching_id: str = Field(
        ..., alias="wellboreMatchingId", description="The matching id of the wellbore related to NPT event."
    )
    npt_code: Optional[str] = Field(
        None, alias="nptCode", description="The NPT code to identify event.", example="ijiefe3:3463:npt"
    )
    npt_code_detail: Optional[str] = Field(
        None, alias="nptCodeDetail", description="A written explanation of an NPT event.", example="leak:242"
    )
    npt_level: Optional[str] = Field(
        None, alias="nptLevel", description="defines the level of non-productive time for the activity.", example="0"
    )
    source: EventSource
    description: Optional[str] = Field(
        None, description="Textual description of the event.", example="factual description of event"
    )
    start_time: Optional[int] = Field(
        None,
        alias="startTime",
        description="epoch time stamp in milliseconds for when the NPT event started.",
        example=1,
    )
    end_time: Optional[int] = Field(
        None, alias="endTime", description="epoch time stamp in milliseconds for when the NPT event ended.", example=1
    )
    measured_depth: Optional[Distance] = Field(None, alias="measuredDepth")
    duration: Optional[float] = Field(None, description="Duration (hours) of the NPT event..", example=6.5)
    root_cause: Optional[str] = Field(
        None,
        alias="rootCause",
        description="Textual description of what caused the NPT event.",
        example="Leakage in pipe",
    )
    location: Optional[str] = Field(
        None, description="Location of NPT event.", example="location where failure event took place"
    )
    subtype: Optional[str] = Field(None, description="subtype of NPT event.", example="some subtype")


class NptIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="The CDF asset external id of the wellbore where the event happened.\nThe asset must have been ingested as a wellbore with the same source\nas this NPT event.\n",
    )
    npt_code: Optional[str] = Field(
        None, alias="nptCode", description="The NPT code to identify event.", example="ijiefe3:3463:npt"
    )
    npt_code_detail: Optional[str] = Field(
        None, alias="nptCodeDetail", description="A written explanation of an NPT event.", example="leak:242"
    )
    npt_level: Optional[str] = Field(
        None, alias="nptLevel", description="defines the level of non-productive time for the activity.", example="0"
    )
    source: EventSource
    description: Optional[str] = Field(
        None, description="Textual description of the event.", example="factual description of event"
    )
    start_time: Optional[int] = Field(
        None, alias="startTime", description="epoch time stamp in milliseconds for when the NPT event started."
    )
    end_time: Optional[int] = Field(
        None, alias="endTime", description="epoch time stamp in milliseconds for when the NPT event ended."
    )
    measured_depth: Optional[Distance] = Field(None, alias="measuredDepth")
    root_cause: Optional[str] = Field(
        None,
        alias="rootCause",
        description="Textual description of what caused the NPT event.",
        example="Leakage in pipe",
    )
    location: Optional[str] = Field(
        None, description="Location of NPT event.", example="location where failure event took place"
    )
    subtype: Optional[str] = Field(None, description="subtype of NPT event.")


class NptIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[NptIngestion] = Field(..., max_items=1000, min_items=1)


class NptItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Npt]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class NptFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measured_depth: Optional[DistanceRange] = Field(None, alias="measuredDepth")
    duration: Optional[DurationRange] = None
    npt_code: Optional[PropertyFilter] = Field(None, alias="nptCode")
    npt_code_detail: Optional[PropertyFilter] = Field(None, alias="nptCodeDetail")
    root_cause: Optional[PropertyFilter] = Field(None, alias="rootCause")
    location: Optional[PropertyFilter] = None
    subtype: Optional[PropertyFilter] = None
    wellbore_ids: Optional[List[Identifier]] = Field(None, alias="wellboreIds", max_items=1000, min_items=1)


class NptFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[NptFilter] = None


class WellNptFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    exists: Optional[bool] = Field(True, description="Set to false to filter on non-existence of particular npt events")
    measured_depth: Optional[DistanceRange] = Field(None, alias="measuredDepth")
    duration: Optional[DurationRange] = None
    npt_codes: Optional[ContainsAllOrAny] = Field(None, alias="nptCodes")
    npt_code_details: Optional[ContainsAllOrAny] = Field(None, alias="nptCodeDetails")


class NptAggregateRequest(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    filter: NptAggregateRequestFilter
    group_by: List[NptAggregateEnum] = Field(..., alias="groupBy")


class NptAggregateRow(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    count: int = Field(..., description="Number of NPT events that matches this group")
    duration: Optional[Duration] = None
    npt_code: Optional[str] = Field(None, alias="nptCode")
    npt_code_detail: Optional[str] = Field(None, alias="nptCodeDetail")


class Nds(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ..., alias="wellboreAssetExternalId", description="The CDF asset external id to the connected wellbore."
    )
    wellbore_matching_id: str = Field(
        ..., alias="wellboreMatchingId", description="The matching id of the wellbore related to NDS event."
    )
    source: EventSource
    description: Optional[str] = Field(
        None, description="Textual description of the event.", example="REAM OUT TIGHT HOLE"
    )
    hole_diameter: Optional[Distance] = Field(None, alias="holeDiameter")
    hole_start: Optional[Distance] = Field(None, alias="holeStart", description="Deprecated field, use holeTop")
    hole_end: Optional[Distance] = Field(None, alias="holeEnd", description="Deprecated field, use holeBase")
    hole_top: Optional[Distance] = Field(None, alias="holeTop")
    hole_base: Optional[Distance] = Field(None, alias="holeBase")
    risk_type: Optional[str] = Field(None, alias="riskType", description="Type of NDS risk.", example="some risk type")
    subtype: Optional[str] = Field(None, description="subtype of NDS risk.", example="some subtype")
    severity: Optional[int] = Field(None, description="severity of NDS event", example=1)
    probability: Optional[int] = Field(None, description="NDS event probability, between 0-5")


class NdsItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Nds]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class NdsIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="The CDF asset external id of the wellbore where the event happened.\nThe asset must have been ingested as a wellbore with the same source\nas this NDS event.\n",
    )
    source: EventSource
    description: Optional[str] = Field(
        None, description="Textual description of the event.", example="REAM OUT TIGHT HOLE"
    )
    hole_diameter: Optional[Distance] = Field(None, alias="holeDiameter")
    hole_start: Optional[Distance] = Field(None, alias="holeStart", description="Deprecated field, use holeTop")
    hole_end: Optional[Distance] = Field(None, alias="holeEnd", description="Deprecated field, use holeBase")
    hole_top: Optional[Distance] = Field(None, alias="holeTop")
    hole_base: Optional[Distance] = Field(None, alias="holeBase")
    severity: Optional[int] = Field(None, description="severity of NDS event", example=1)
    risk_type: Optional[str] = Field(None, alias="riskType", description="Type of NDS risk.", example="some risk type")
    subtype: Optional[str] = Field(None, description="subtype of NDS risk.", example="some subtype")
    probability: Optional[int] = Field(None, description="NDS event probability, between 0-5")


class NdsIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[NdsIngestion] = Field(..., max_items=1000, min_items=1)
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class NdsFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    hole_start: Optional[DistanceRange] = Field(None, alias="holeStart", description="Deprecated field, use holeTop")
    hole_end: Optional[DistanceRange] = Field(None, alias="holeEnd", description="Deprecated field, use holeBase")
    hole_top: Optional[DistanceRange] = Field(None, alias="holeTop")
    hole_base: Optional[DistanceRange] = Field(None, alias="holeBase")
    probabilities: Optional[List[int]] = None
    severities: Optional[List[int]] = None
    risk_type: Optional[PropertyFilter] = Field(None, alias="riskType")
    subtype: Optional[PropertyFilter] = None
    wellbore_ids: Optional[List[Identifier]] = Field(None, alias="wellboreIds", max_items=1000, min_items=1)


class NdsFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[NdsFilter] = None


class WellNdsFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    exists: Optional[bool] = Field(True, description="Set to false to filter on non-existence of particular nds events")
    severities: Optional[ContainsAllOrAnyInt] = None
    probabilities: Optional[ContainsAllOrAnyInt] = None
    risk_types: Optional[ContainsAllOrAny] = Field(None, alias="riskTypes")


class NdsAggregateRequest(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    filter: NdsAggregateRequestFilter
    group_by: List[NdsAggregateEnum] = Field(..., alias="groupBy")


class NdsAggregate(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_matching_id: str = Field(..., alias="wellboreMatchingId")
    items: List[NdsAggregateRow] = Field(..., min_items=1)


class DepthIndexColumn(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    column_external_id: str = Field(
        ..., alias="columnExternalId", description="External id of the sequence column", example="MD"
    )
    unit: DistanceUnit
    type: DepthIndexTypeEnum


class MeasurementTypeItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[MeasurementType]


class TimeMeasurement(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: str
    source_name: str = Field(..., alias="sourceName")
    external_id: str = Field(..., alias="externalId")
    description: Optional[str] = None
    wellbore_asset_external_id: str = Field(..., alias="wellboreAssetExternalId")
    columns: List[TimeMeasurementColumn]


class TimeMeasurementIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[TimeMeasurement] = Field(..., max_items=1000, min_items=1)


class TimeMeasurementItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[TimeMeasurement] = Field(..., max_items=1000)
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class TimeMeasurementFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[TimeMeasurementFilter] = None


class TrajectoryIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="The CDF asset external id of the wellbore where the event happened.\nThe asset must have been ingested as a wellbore with the same source\nas this NPT event.\n",
    )
    source: SequenceSource
    type: Optional[str] = None
    measured_depth_unit: DistanceUnitEnum = Field(..., alias="measuredDepthUnit")
    inclination_unit: AngleUnitEnum = Field(..., alias="inclinationUnit")
    azimuth_unit: AngleUnitEnum = Field(..., alias="azimuthUnit")
    dogleg_severity_unit: Optional[DoglegSeverityUnit] = Field(None, alias="doglegSeverityUnit")
    is_definitive: bool = Field(..., alias="isDefinitive")
    rows: List[TrajectoryIngestionRow]


class WellTopSurfaceIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: str = Field(..., description="The name of the lithostratigraphic unit")
    top_measured_depth: float = Field(
        ..., alias="topMeasuredDepth", description="The measured depth of the top of the lithostratigraphic unit"
    )
    base_measured_depth: Optional[float] = Field(
        None, alias="baseMeasuredDepth", description="The measured depth of the base of the lithostratigraphic unit."
    )
    lithostratigraphic: Optional[Lithostratigraphic] = None
    chronostratigraphic: Optional[Chronostratigraphic] = None


class WellTopSurface(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    name: str = Field(..., description="The name of the lithostratigraphic unit")
    top: WellTopSurfaceDepth = Field(..., description="The top surface of the lithostratigraphic unit")
    base: Optional[WellTopSurfaceDepth] = Field(
        None, description="The base of the lithostratigraphic unit, if available"
    )
    lithostratigraphic: Optional[Lithostratigraphic] = None
    chronostratigraphic: Optional[Chronostratigraphic] = None


class WellTopsFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[WellTopsFilter] = None


class DoglegSeverity(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    unit: DoglegSeverityUnit
    value: float


class DoglegSeverityRange(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    unit: DoglegSeverityUnit
    min: Optional[float] = None
    max: Optional[float] = None


class TrajectoryData(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(..., alias="wellboreAssetExternalId")
    wellbore_matching_id: str = Field(
        ..., alias="wellboreMatchingId", description="The matching id of the wellbore related to trajectory data."
    )
    source: SequenceSource
    type: Optional[str] = None
    measured_depth_unit: DistanceUnitEnum = Field(..., alias="measuredDepthUnit")
    inclination_unit: AngleUnitEnum = Field(..., alias="inclinationUnit")
    azimuth_unit: AngleUnitEnum = Field(..., alias="azimuthUnit")
    true_vertical_depth_unit: DistanceUnitEnum = Field(..., alias="trueVerticalDepthUnit")
    equivalent_departure_unit: DistanceUnitEnum = Field(..., alias="equivalentDepartureUnit")
    offset_unit: DistanceUnitEnum = Field(..., alias="offsetUnit")
    dogleg_severity_unit: DoglegSeverityUnit = Field(..., alias="doglegSeverityUnit")
    dogleg_severity_is_computed: bool = Field(
        ...,
        alias="doglegSeverityIsComputed",
        description="Indicates whether the dogleg severities in this trajectory were computed by the well service (true), or whether it was provided during ingestion (false)",
    )
    rows: List[TrajectoryDataRow]


class TrajectoryDataItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[TrajectoryData]


class TrajectoryInterpolationRequest(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_id: Identifier = Field(
        ..., alias="wellboreId", description="Interpolate the definitive trajectory of this wellbore"
    )
    measured_depths: List[float] = Field(..., alias="measuredDepths")
    measured_depth_unit: DistanceUnit = Field(..., alias="measuredDepthUnit")
    true_vertical_depth_unit: Optional[DistanceUnit] = Field(None, alias="trueVerticalDepthUnit")


class TrueVerticalDepths(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    true_vertical_depths: List[float] = Field(..., alias="trueVerticalDepths")
    measured_depths: List[float] = Field(..., alias="measuredDepths")
    true_vertical_depth_unit: DistanceUnit = Field(..., alias="trueVerticalDepthUnit")
    sequence_source: SequenceSource = Field(..., alias="sequenceSource")
    wellbore_asset_external_id: str = Field(..., alias="wellboreAssetExternalId")
    wellbore_matching_id: str = Field(..., alias="wellboreMatchingId")


class CasingAssembly(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    min_inside_diameter: Distance = Field(..., alias="minInsideDiameter")
    min_outside_diameter: Distance = Field(..., alias="minOutsideDiameter")
    max_outside_diameter: Distance = Field(..., alias="maxOutsideDiameter")
    original_measured_depth_top: Distance = Field(..., alias="originalMeasuredDepthTop")
    original_measured_depth_base: Distance = Field(..., alias="originalMeasuredDepthBase")
    type: Optional[str] = None
    report_description: Optional[str] = Field(None, alias="reportDescription")


class CasingSchematicIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(..., alias="wellboreAssetExternalId")
    casing_assemblies: List[CasingAssembly] = Field(..., alias="casingAssemblies")
    source: SequenceSource
    phase: str


class CasingSchematic(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(..., alias="wellboreAssetExternalId")
    wellbore_matching_id: str = Field(
        ..., alias="wellboreMatchingId", description="The matching id of the wellbore related to CasingSchematic."
    )
    casing_assemblies: List[CasingAssembly] = Field(..., alias="casingAssemblies")
    source: SequenceSource
    phase: str


class HoleSectionIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    source: GeneralExternalId
    name: str = Field(..., description="Name for the Hole Section.")
    hole_size: float = Field(
        ..., alias="holeSize", description="Hole size of the hole section in hole section group hole size unit."
    )
    bit_size: float = Field(
        ..., alias="bitSize", description="Bit size of the hole section in hole section group bit size unit."
    )
    start_time: Optional[int] = Field(
        None,
        alias="startTime",
        description="The timestamp in milliseconds since the epoch (Jan 1, 1970) for when the hole sections was started.",
        example=1,
    )
    end_time: Optional[int] = Field(
        None,
        alias="endTime",
        description="The timestamp in milliseconds since the epoch (Jan 1, 1970) for when the hole sections was completed.",
        example=1,
    )
    top_measured_depth: float = Field(
        ..., alias="topMeasuredDepth", description="Top of the hole section - shallowest point by measured depth."
    )
    base_measured_depth: float = Field(
        ..., alias="baseMeasuredDepth", description="Base of the hole section - deepest point by measured depth."
    )


class HoleSection(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    source: GeneralExternalId
    name: str = Field(..., description="Name for the Hole Section.")
    hole_size: Optional[float] = Field(
        None, alias="holeSize", description="Hole size of the hole section in hole section group hole size unit."
    )
    bit_size: Optional[float] = Field(
        None, alias="bitSize", description="Bit size of the hole section in hole section group bit size unit."
    )
    start_time: Optional[int] = Field(
        None,
        alias="startTime",
        description="The timestamp in milliseconds since the epoch (Jan 1, 1970) for when the hole sections was started.",
        example=1,
    )
    end_time: Optional[int] = Field(
        None,
        alias="endTime",
        description="The timestamp in milliseconds since the epoch (Jan 1, 1970) for when the hole sections was completed.",
        example=1,
    )
    top_measured_depth: Optional[float] = Field(
        None, alias="topMeasuredDepth", description="Top of the hole section - shallowest point by measured depth."
    )
    base_measured_depth: Optional[float] = Field(
        None, alias="baseMeasuredDepth", description="Base of the hole section - deepest point by measured depth."
    )


class HoleSectionFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_ids: Optional[List[Identifier]] = Field(
        None,
        alias="wellboreIds",
        description="Filter hole sections that are related to one of given wellbores.",
        max_items=1000,
        min_items=1,
    )
    bit_size: Optional[Distance] = Field(
        None, alias="bitSize", description="Filter hole sections with a specific hole size.", title="Distance"
    )
    hole_size: Optional[Distance] = Field(
        None, alias="holeSize", description="Filter hole sections with a specific bit size.", title="Distance"
    )
    top_measured_depth: Optional[DistanceRange] = Field(
        None,
        alias="topMeasuredDepth",
        description="Filter hole sections with hole top in a given measured depth range.",
        title="DistanceRange",
    )
    base_measured_depth: Optional[DistanceRange] = Field(
        None,
        alias="baseMeasuredDepth",
        description="Filter hole sections with hole base in a given measured depth range.",
        title="DistanceRange",
    )
    start_time: Optional[TimeRange] = Field(
        None,
        alias="startTime",
        description="Filter hole sections with construction start time within a given time range.",
        title="DistanceRange",
    )
    end_time: Optional[TimeRange] = Field(
        None,
        alias="endTime",
        description="Filter hole sections with construction end time within a given time range.",
        title="DistanceRange",
    )
    is_definitive: Optional[bool] = Field(
        None,
        alias="isDefinitive",
        description="Specifies if the query is to return only definitive or not. If value is not set, then all hole sections are returned.",
    )


class HoleSectionFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[HoleSectionFilter] = None


class WellTrajectoryFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    max_measured_depth: Optional[DistanceRange] = Field(None, alias="maxMeasuredDepth")
    max_true_vertical_depth: Optional[DistanceRange] = Field(None, alias="maxTrueVerticalDepth")
    max_inclination: Optional[AngleRange] = Field(None, alias="maxInclination")
    max_dogleg_severity: Optional[DoglegSeverityRange] = Field(None, alias="maxDoglegSeverity")


class PolygonFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    geometry: str = Field(..., example="POINT (1.42 1.35)")
    crs: str = Field(..., example="EPSG:4326")
    geometry_type: Optional[GeometryTypeEnum] = Field(None, alias="geometryType")


class WellDepthMeasurementFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measurement_types: Optional[ContainsAllOrAnyMeasurementType] = Field(None, alias="measurementTypes")
    measured_depth: Optional[DistanceRange] = Field(None, alias="measuredDepth")
    true_vertical_depth: Optional[DistanceRange] = Field(None, alias="trueVerticalDepth")


class WellTimeMeasurementFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measurement_types: Optional[ContainsAllOrAnyMeasurementType] = Field(None, alias="measurementTypes")


class WellHoleSectionFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    hole_size: Optional[Distance] = Field(None, alias="holeSize")
    bit_size: Optional[Distance] = Field(None, alias="bitSize")
    exists: Optional[bool] = Field(
        True, description="Set to false to filter on wellbores that do not have registered hole sections\n"
    )


class DepthMeasurementDataRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    sequence_external_id: str = Field(..., alias="sequenceExternalId")
    measured_depth: Optional[DistanceRange] = Field(None, alias="measuredDepth")
    depth_unit: Optional[DistanceUnit] = Field(None, alias="depthUnit")
    measurement_types: Optional[List[str]] = Field(None, alias="measurementTypes")
    column_external_ids: Optional[List[str]] = Field(None, alias="columnExternalIds")
    top_surface_name: Optional[str] = Field(None, alias="topSurfaceName")


class DepthMeasurementHoleSectionFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    bit_size: Optional[Distance] = Field(None, alias="bitSize")
    hole_size: Optional[Distance] = Field(None, alias="holeSize")


class DepthMeasurementDataColumn(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    external_id: str = Field(
        ...,
        alias="externalId",
        description="User provided column identifier (Unique for a given sequence)",
        example="DPS1",
    )
    name: Optional[str] = Field(None, description="Human readable name of the column", example="Depth sensor 1")
    measurement_type: str = Field(..., alias="measurementType")
    unit: str = Field(..., description="Unit for the measurement type", example="meter")
    value_type: DepthMeasurementValueTypeEnum = Field(..., alias="valueType")


class DeleteExternalIdSource(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[ExternalIdSource] = Field(
        ..., description="List of externalIds to delete.", max_items=1000, min_items=1
    )


class DeleteSources(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Source] = Field(..., max_items=1000, min_items=1)
    recursive: Optional[bool] = Field(
        False,
        description="If set to true, it will delete all data under the source.\nIf set to false, it will fail if one of the sources has any wells.",
    )


class SourceItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Source] = Field(..., max_items=1000)


class MnemonicMatch(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    company_name: str = Field(..., alias="companyName")
    measurement_type: str = Field(..., alias="measurementType", example="measured depth")
    primary_quantity_class: Optional[str] = Field(None, alias="primaryQuantityClass")
    tools: List[MnemonicMatchTool]


class MeasurementTypeFilterRequest(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[MeasurementTypeFilter] = None


class MeasurementTypeDetailsItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[MeasurementTypeDetails]


class Datum(Distance):
    class Config:
        allow_population_by_field_name = True

    reference: str = Field(..., example="KB")


class WellIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellIngestion] = Field(..., max_items=1000, min_items=1)


class WellWellheadViewItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellWellheadView]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class Wellbore(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    matching_id: str = Field(..., alias="matchingId")
    name: str
    description: Optional[str] = None
    well_matching_id: str = Field(..., alias="wellMatchingId")
    parent_wellbore_matching_id: Optional[str] = Field(
        None, alias="parentWellboreMatchingId", description="Parent wellbore if it exists"
    )
    sources: List[AssetSource]
    datum: Optional[Datum] = None


class WellboreIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    matching_id: Optional[str] = Field(None, alias="matchingId")
    name: str
    description: Optional[str] = None
    well_asset_external_id: str = Field(
        ..., alias="wellAssetExternalId", description="The CDF assetExternalId of the well parent of this wellbore"
    )
    parent_wellbore_asset_external_id: Optional[str] = Field(
        None,
        alias="parentWellboreAssetExternalId",
        description="If this wellbore is below another wellbore, add the parent wellbore's CDF assetExternalId here",
    )
    source: AssetSource
    datum: Optional[Datum] = None


class WellboreItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Wellbore]


class WellboreMergeDetails(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore: Wellbore
    field_sources: FieldSources1 = Field(..., alias="fieldSources")


class WellboreMergeDetailItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellboreMergeDetails]


class NptAggregate(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_matching_id: str = Field(..., alias="wellboreMatchingId")
    items: List[NptAggregateRow] = Field(..., min_items=1)


class NdsAggregateItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[NdsAggregate]


class DepthMeasurement(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="The CDF asset external id of the wellbore.\nThe asset must have been ingested as a wellbore with the same source\nas this NPT event.\n",
    )
    wellbore_matching_id: str = Field(
        ..., alias="wellboreMatchingId", description="The matching id of the wellbore related to depth measurement."
    )
    source: SequenceSource
    depth_column: DepthIndexColumn = Field(..., alias="depthColumn")
    datum: Optional[Datum] = None
    columns: List[DepthMeasurementColumn]
    run: Optional[str] = None
    producer: Optional[str] = None
    start_time: Optional[int] = Field(
        None,
        alias="startTime",
        description="The time the depth measurement started. The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.",
    )
    end_time: Optional[int] = Field(
        None,
        alias="endTime",
        description="The time the depth measurement completed. The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.",
    )
    depth_range: Optional[DistanceRange] = Field(None, alias="depthRange")


class DepthMeasurementIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="The CDF asset external id of the wellbore.\nThe asset must have been ingested as a wellbore with the same source\nas this NPT event.\n",
    )
    source: SequenceSource
    depth_column: DepthIndexColumn = Field(..., alias="depthColumn")
    datum: Optional[Datum] = None
    columns: List[DepthMeasurementColumn]
    run: Optional[str] = None
    producer: Optional[str] = None
    start_time: Optional[int] = Field(
        None,
        alias="startTime",
        description="The time the depth measurement started. The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.",
    )
    end_time: Optional[int] = Field(
        None,
        alias="endTime",
        description="The time the depth measurement completed. The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time (UTC), minus leap seconds.",
    )


class DepthMeasurementItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[DepthMeasurement]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class DepthMeasurementIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[DepthMeasurementIngestion] = Field(..., max_items=1000, min_items=1)


class Trajectory(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ..., alias="wellboreAssetExternalId", description="The CDF asset external id to the connected wellbore."
    )
    wellbore_matching_id: str = Field(
        ..., alias="wellboreMatchingId", description="The matching id of the connected wellbore."
    )
    type: Optional[str] = None
    max_measured_depth: float = Field(..., alias="maxMeasuredDepth")
    max_true_vertical_depth: float = Field(..., alias="maxTrueVerticalDepth")
    max_inclination: float = Field(..., alias="maxInclination")
    max_dogleg_severity: DoglegSeverity = Field(..., alias="maxDoglegSeverity")
    source: SequenceSource
    is_definitive: bool = Field(..., alias="isDefinitive")


class TrajectoryItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Trajectory]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class TrajectoryIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[TrajectoryIngestion] = Field(..., max_items=1000, min_items=1)


class WellTopsIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="External id for the wellbore asset the well tops are associated with. Source for the wellbore and for the well tops must be the same.\n",
    )
    source: SequenceSource
    measured_depth_unit: DistanceUnitEnum = Field(..., alias="measuredDepthUnit")
    tops: List[WellTopSurfaceIngestion] = Field(..., description="A list of well tops to ingest")


class WellTops(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    wellbore_matching_id: str = Field(
        ...,
        alias="wellboreMatchingId",
        description="The matching id of the wellbore containing these lithostratigraphic units.",
    )
    wellbore_name: str = Field(
        ..., alias="wellboreName", description="The name of the wellbore containing these lithostratigraphic units."
    )
    source: SequenceSource
    measured_depth_unit: DistanceUnitEnum = Field(..., alias="measuredDepthUnit")
    true_vertical_depth_unit: DistanceUnitEnum = Field(..., alias="trueVerticalDepthUnit")
    tops: List[WellTopSurface]


class TrajectoryInterpolationRequestItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[TrajectoryInterpolationRequest] = Field(..., max_items=1000, min_items=1)
    ignore_missing_trajectories: Optional[bool] = Field(None, alias="ignoreMissingTrajectories")
    ignore_unknown_measured_depths: Optional[bool] = Field(None, alias="ignoreUnknownMeasuredDepths")


class TrajectoryInterpolationItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[TrueVerticalDepths]


class CasingIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[CasingSchematicIngestion] = Field(..., max_items=1000, min_items=1)


class CasingItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[CasingSchematic]
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class HoleSectionGroupIngestion(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    source_name: str = Field(..., alias="sourceName", description="Name of the source for ingested hole sections.")
    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="External id for the wellbore asset this hole section group is associated with. Source for the wellbore and for the hole section group must be the same.",
    )
    measured_depth_unit: DistanceUnitEnum = Field(..., alias="measuredDepthUnit")
    size_unit: DistanceUnitEnum = Field(..., alias="sizeUnit")
    external_id: str = Field(
        ...,
        alias="externalId",
        description="A project unique identifier for the hole section group. This id will be used to reference this hole section group.",
    )
    is_definitive: Optional[bool] = Field(
        False,
        alias="isDefinitive",
        description="If true, this hole section group will become a new definite hole section group for the matched wellbore.",
    )
    sections: List[HoleSectionIngestion] = Field(..., description="A list of hole sections for this group.")


class HoleSectionGroup(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    source_name: str = Field(..., alias="sourceName", description="Name of the source for holes section group.")
    wellbore_asset_external_id: str = Field(
        ...,
        alias="wellboreAssetExternalId",
        description="External id for the wellbore asset this hole section group is associated with.",
    )
    wellbore_matching_id: str = Field(
        ...,
        alias="wellboreMatchingId",
        description="The matching id of the wellbore this hole section group is associated with.",
    )
    measured_depth_unit: DistanceUnitEnum = Field(..., alias="measuredDepthUnit")
    size_unit: DistanceUnitEnum = Field(..., alias="sizeUnit")
    external_id: str = Field(
        ...,
        alias="externalId",
        description="A project unique identifier for the hole section group. This id is used to reference this hole section group.",
    )
    is_definitive: Optional[bool] = Field(
        None,
        alias="isDefinitive",
        description="Describes if the hole section group is definite. A definite hole section group is a group that contains information about the actual hole section for a given wellbore.",
    )
    sections: List[HoleSection] = Field(..., description="A list of hole sections for this group.")


class WellFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    quadrant: Optional[PropertyFilter] = None
    region: Optional[PropertyFilter] = None
    block: Optional[PropertyFilter] = None
    field: Optional[PropertyFilter] = None
    operator: Optional[PropertyFilter] = None
    well_type: Optional[PropertyFilter] = Field(None, alias="wellType")
    license: Optional[PropertyFilter] = None
    sources: Optional[List[str]] = Field(None, example=["EDM", "Sitecom"])
    water_depth: Optional[DistanceRange] = Field(None, alias="waterDepth")
    datum: Optional[DistanceRange] = None
    spud_date: Optional[DateRange] = Field(None, alias="spudDate")
    polygon: Optional[PolygonFilter] = None
    trajectories: Optional[WellTrajectoryFilter] = None
    depth_measurements: Optional[WellDepthMeasurementFilter] = Field(None, alias="depthMeasurements")
    time_measurements: Optional[WellTimeMeasurementFilter] = Field(None, alias="timeMeasurements")
    npt: Optional[WellNptFilter] = None
    nds: Optional[WellNdsFilter] = None
    well_tops: Optional[WellTopFilter] = Field(None, alias="wellTops")
    casings: Optional[WellCasingFilter] = None
    hole_sections: Optional[WellHoleSectionFilter] = Field(None, alias="holeSections")
    data_availability: Optional[WellDataAvailabilityFilter] = Field(None, alias="dataAvailability")


class WellFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[WellFilter] = None
    search: Optional[WellSearch] = None
    output_crs: Optional[str] = Field(None, alias="outputCrs", example="EPSG:4326")
    aggregates: Optional[List[WellFilterAggregatesEnum]] = None


class DepthMeasurementDataRequestItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[DepthMeasurementDataRequest]


class DepthMeasurementFilter(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    measurement_types: Optional[List[str]] = Field(None, alias="measurementTypes")
    wellbore_ids: Optional[List[Identifier]] = Field(None, alias="wellboreIds", max_items=1000, min_items=1)
    hole_sections: Optional[DepthMeasurementHoleSectionFilter] = Field(None, alias="holeSections")


class DepthMeasurementData(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    id: int
    source: SequenceSource
    depth_column: DepthIndexColumn = Field(..., alias="depthColumn")
    depth_unit: DistanceUnit = Field(..., alias="depthUnit")
    columns: List[DepthMeasurementDataColumn] = Field(..., description="Column information in order given by data")
    rows: List[DepthMeasurementRow] = Field(..., description="List of row information")
    next_cursor: Optional[str] = Field(
        None, alias="nextCursor", description="Cursor to get the next page of results (if available)."
    )


class MnemonicMatchGroup(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    mnemonic: str = Field(..., example="DEPT")
    matches: List[MnemonicMatch]


class Well(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    matching_id: str = Field(..., alias="matchingId")
    name: str = Field(..., example="25/5-5")
    description: Optional[str] = Field(None, description="additional description for the data")
    country: Optional[str] = Field(None, example="Norway")
    quadrant: Optional[str] = Field(None, example="25")
    region: Optional[str] = None
    block: Optional[str] = Field(None, example="25/5")
    field: Optional[str] = None
    operator: Optional[str] = None
    spud_date: Optional[date] = Field(None, alias="spudDate")
    well_type: Optional[str] = Field(None, alias="wellType", description='For example "exploration" or "development"')
    license: Optional[str] = Field(None, description="Well license")
    wellhead: Wellhead
    water_depth: Optional[Distance] = Field(None, alias="waterDepth")
    sources: List[AssetSource] = Field(
        ...,
        description="List of source systems that are associated to this well (user may drill down further to explore metadata for each source by calling well/{id}/source/{sourcename}",
    )
    wellbores: Optional[List[Wellbore]] = Field(None, description="List of wellbores associated to this well")


class WellItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[Well]
    wells_count: Optional[int] = Field(None, alias="wellsCount")
    wellbores_count: Optional[int] = Field(None, alias="wellboresCount")
    next_cursor: Optional[str] = Field(None, alias="nextCursor")


class WellMergeDetails(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    well: Well
    field_sources: FieldSources = Field(..., alias="fieldSources")


class WellMergeDetailItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellMergeDetails]


class WellboreIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellboreIngestion] = Field(..., max_items=1000, min_items=1)


class NptAggregateItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[NptAggregate]


class WellTopIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellTopsIngestion] = Field(..., max_items=1000, min_items=1)


class WellTopItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[WellTops]
    next_cursor: Optional[str] = Field(
        None, alias="nextCursor", description="Cursor to get the next page of results (if available)."
    )


class HoleSectionGroupsIngestionItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[HoleSectionGroupIngestion] = Field(..., max_items=1000, min_items=1)


class HoleSectionGroupsItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[HoleSectionGroup] = Field(..., min_items=0)
    next_cursor: Optional[str] = Field(
        None, alias="nextCursor", description="The cursor to get the next page of results (if available)."
    )


class DepthMeasurementFilterRequest(CursorAndLimit):
    class Config:
        allow_population_by_field_name = True

    filter: Optional[DepthMeasurementFilter] = None


class DepthMeasurementDataItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[DepthMeasurementData]


class MnemonicMatchItems(WellsBaseModel):
    class Config:
        allow_population_by_field_name = True

    items: List[MnemonicMatchGroup]
